/* Copyright (C) 1991-1993, 1995-2001, 2002, 2006 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

/*
 *	ISO C99 Standard: 7.21 String handling	<string.h>
 */

#ifndef	_STRING_H
#define	_STRING_H	1

#include <features.h>

__BEGIN_DECLS

/* Get size_t and NULL from <stddef.h>.  */
#define	__need_size_t
#define	__need_NULL
#include <stddef.h>


__BEGIN_NAMESPACE_STD
#ifndef __SYSC__
/* Copy N bytes of SRC to DEST.  */
extern void *memcpy (void *__restrict __dest,
		     __const void *__restrict __src, size_t __n) __THROW;
#endif /* !__SYSC__ */
/* Copy N bytes of SRC to DEST, guaranteeing
   correct behavior for overlapping strings.  */
extern void *memmove (void *__dest, __const void *__src, size_t __n)
     __THROW;
__END_NAMESPACE_STD

/* Copy no more than N bytes of SRC to DEST, stopping when C is found.
   Return the position in DEST one byte past where C was copied,
   or NULL if C was not found in the first N bytes of SRC.  */
#if defined __USE_SVID || defined __USE_BSD || defined __USE_XOPEN
extern void *memccpy (void *__restrict __dest, __const void *__restrict __src,
		      int __c, size_t __n)
     __THROW;
#endif /* SVID.  */

#if defined _TPF_SOURCE && !defined __SYSC__
/* Inline version of strnlen for TPF performance reasons.  */
extern inline size_t strnlen( char *_str, int _limit ) {
    long  __l = (long) _limit;
    char *__s = _str;
    if( !__l || !__s )              /* Reject null parms.  */
       return 0;
    asm volatile (
         "xgr %%r0,%%r0\n\t"
         "la %0,0(%1,%0)\n\t"
         "lgr %%r1,%1\n\t"
         "srst %0,%1\n\t"
         "jo .-4\n\t"
         "lgr %1,%%r1\n\t"
         "sgr %0,%1\n\t"
         : "+&a" (__l),  "+&a" (__s)
         :
         : "r0", "r1", "memory", "cc" );
    return __l;
 }
#else
  #ifdef        __USE_GNU
  /* Find the length of STRING, but scan at most MAXLEN characters.
     If no '\0' terminator is found in that many characters, return MAXLEN.  */
  extern size_t strnlen (__const char *__string, size_t __maxlen)
       __THROW __attribute_pure__;
  #endif  /* __USE_GNU  */
#endif    /* _TPF_SOURCE && __SYSC__ */


#ifndef __SYSC__
__BEGIN_NAMESPACE_STD
/* Set N bytes of S to C.  */
extern void *memset (void *__s, int __c, size_t __n) __THROW;

/* Compare N bytes of S1 and S2.  */
extern int memcmp (__const void *__s1, __const void *__s2, size_t __n)
     __THROW __attribute_pure__;

/* Search N bytes of S for C.  */
extern void *memchr (__const void *__s, int __c, size_t __n)
      __THROW __attribute_pure__;
__END_NAMESPACE_STD
#endif

#ifdef __USE_GNU
/* Search in S for C.  This is similar to `memchr' but there is no
   length limit.  */
extern void *rawmemchr (__const void *__s, int __c) __THROW __attribute_pure__;

/* Search N bytes of S for the final occurrence of C.  */
extern void *memrchr (__const void *__s, int __c, size_t __n)
      __THROW __attribute_pure__;
#endif


__BEGIN_NAMESPACE_STD
/* Copy SRC to DEST.  */
#if defined _TPF_SOURCE && !defined __SYSC__
extern inline char *strcpy( char *__restrict __dest,
                              __const char *__restrict __src ) __THROW {
   __builtin_memcpy(__dest, __src, __builtin_strlen(__src) + 1);
   return __dest;
}
extern inline char *strncpy( char *__restrict __dest,
                              __const char *__restrict __src, size_t __n ) __THROW {
   size_t sl = strnlen((char*)__src, __n);
   if( sl < __n ) {
      __builtin_memcpy(__dest, __src, sl );
      __builtin_memset(__dest+sl, '\0', __n - sl);
   }
   else
     __builtin_memcpy(__dest, __src, __n );
   return __dest;
}
#elif !defined __SYSC__
extern char *strcpy (char *__restrict __dest, __const char *__restrict __src)
     __THROW;
/* Copy no more than N characters of SRC to DEST.  */
extern char *strncpy (char *__restrict __dest,
		      __const char *__restrict __src, size_t __n) __THROW;
#endif

#if defined _TPF_SOURCE && !defined __SYSC__
/* Append SRC onto DEST.  */
extern inline char *strcat( char *__restrict __dest,
                            const char *__restrict __src ) __THROW {
   __builtin_memcpy( (__dest+__builtin_strlen( __dest )), __src,
                      __builtin_strlen( __src ) + 1 );
   return __dest;
}
/* Append no more than N characters from SRC onto DEST.  */
extern inline char *strncat( char *__restrict __dest,
                             const char *__restrict __src, size_t __n ) __THROW {
   size_t dl = __builtin_strlen( __dest );
   size_t sl = strnlen( (char*)__src, __n );
   if( sl < __n )
     __builtin_memcpy( (__dest+dl), __src, sl + 1 );
   else {
     __builtin_memcpy( (__dest+dl), __src, __n );
     *(__dest+dl+__n) = 0;
   }
   return __dest;
}
#elif !defined __SYSC__
/* Append SRC onto DEST.  */
extern char *strcat (char *__restrict __dest, __const char *__restrict __src)
     __THROW;
/*  no more than N characters from SRC onto DEST.  */
extern char *strncat (char *__restrict __dest, __const char *__restrict __src,
		      size_t __n) __THROW;
#endif

#if defined _TPF_SOURCE && !defined __SYSC__
/* Compare S1 and S2.  */
extern inline int strcmp (__const char *__s1, __const char *__s2) __THROW {
   size_t len1 = __builtin_strlen(__s1);
   size_t len2 = __builtin_strlen(__s2);
   int rc = __builtin_memcmp( __s1, __s2, (len1 < len2 ? len1 : len2) );
   if( len1 == len2 || rc != 0 )
      return rc;
   else if ( len1 < len2 )
      return -1;
   return 1;
}
/* Compare N characters of S1 and S2.  */
extern inline int strncmp (__const char *__s1, __const char *__s2, size_t __n) __THROW {
   size_t len1 = strnlen((char*)__s1, __n) + 1;
   size_t len2 = strnlen((char*)__s2, __n) + 1;
   return __builtin_memcmp(__s1, __s2, (len1 < len2 ? (len1 < __n ? len1 : __n)
                                                    : (len2 < __n ? len2 : __n)) );
}
#elif !defined __SYSC__
/* Compare S1 and S2.  */
extern int strcmp (__const char *__s1, __const char *__s2)
     __THROW __attribute_pure__;
/* Compare N characters of S1 and S2.  */
extern int strncmp (__const char *__s1, __const char *__s2, size_t __n)
     __THROW __attribute_pure__;
#endif

/* Compare the collated forms of S1 and S2.  */
extern int strcoll (__const char *__s1, __const char *__s2)
     __THROW __attribute_pure__;
/* Put a transformation of SRC into no more than N bytes of DEST.  */
extern size_t strxfrm (char *__restrict __dest,
		       __const char *__restrict __src, size_t __n) __THROW;
__END_NAMESPACE_STD

#ifdef __USE_GNU
/* The following functions are equivalent to the both above but they
   take the locale they use for the collation as an extra argument.
   This is not standardsized but something like will come.  */
# include <xlocale.h>

/* Compare the collated forms of S1 and S2 using rules from L.  */
extern int strcoll_l (__const char *__s1, __const char *__s2, __locale_t __l)
     __THROW __attribute_pure__;
/* Put a transformation of SRC into no more than N bytes of DEST.  */
extern size_t strxfrm_l (char *__dest, __const char *__src, size_t __n,
			 __locale_t __l) __THROW;
#endif

#if defined __USE_SVID || defined __USE_BSD || defined __USE_XOPEN_EXTENDED
/* Duplicate S, returning an identical malloc'd string.  */
extern char *strdup (__const char *__s) __THROW __attribute_malloc__;
#endif

/* Return a malloc'd copy of at most N bytes of STRING.  The
   resultant string is terminated even if no null terminator
   appears before STRING[N].  */
#if defined __USE_GNU
extern char *strndup (__const char *__string, size_t __n)
     __THROW __attribute_malloc__;
#endif

#if defined __USE_GNU && defined __GNUC__
/* Duplicate S, returning an identical alloca'd string.  */
# define strdupa(s)							      \
  (__extension__							      \
    ({									      \
      __const char *__old = (s);					      \
      size_t __len = strlen (__old) + 1;				      \
      char *__new = (char *) __builtin_alloca (__len);			      \
      (char *) memcpy (__new, __old, __len);				      \
    }))

/* Return an alloca'd copy of at most N bytes of string.  */
# define strndupa(s, n)							      \
  (__extension__							      \
    ({									      \
      __const char *__old = (s);					      \
      size_t __len = strnlen (__old, (n));				      \
      char *__new = (char *) __builtin_alloca (__len + 1);		      \
      __new[__len] = '\0';						      \
      (char *) memcpy (__new, __old, __len);				      \
    }))
#endif

__BEGIN_NAMESPACE_STD
#if defined _TPF_SOURCE  && !defined __SYSC__
/* Find the first occurrence of C in S.  */
extern inline char *strchr (__const char *__s, int __c) __THROW {
   return (char *)memchr((void *)__s, __c, __builtin_strlen(__s) + 1);
}
/* Find the last occurrence of C in S.  */
extern inline char *strrchr (__const char *__s, int __c) __THROW {
   unsigned char ch = __c;
   int __i;                                                                     //PJ45408
   for (__i=__builtin_strlen(__s); __i >= 0; --__i)                             //PJ45408
      if (*(__s+__i) == ch ) return (char *)(__s+__i);                          //PJ45408
   return(NULL);

}
#elif !defined __SYSC__
/* Find the first occurrence of C in S.  */
extern char *strchr (__const char *__s, int __c) __THROW __attribute_pure__;
/* Find the last occurrence of C in S.  */
extern char *strrchr (__const char *__s, int __c) __THROW __attribute_pure__;
#endif
__END_NAMESPACE_STD

#ifdef __USE_GNU
/* This function is similar to `strchr'.  But it returns a pointer to
   the closing NUL byte in case C is not found in S.  */
extern char *strchrnul (__const char *__s, int __c) __THROW __attribute_pure__;
#endif

__BEGIN_NAMESPACE_STD
/* Return the length of the initial segment of S which
   consists entirely of characters not in REJECT.  */
extern size_t strcspn (__const char *__s, __const char *__reject)
     __THROW __attribute_pure__;
/* Return the length of the initial segment of S which
   consists entirely of characters in ACCEPT.  */
extern size_t strspn (__const char *__s, __const char *__accept)
     __THROW __attribute_pure__;
#ifndef __SYSC__
/* Find the first occurrence in S of any character in ACCEPT.  */
extern char *strpbrk (__const char *__s, __const char *__accept)
     __THROW __attribute_pure__;
#endif
/* Find the first occurrence of NEEDLE in HAYSTACK.  */
extern char *strstr (__const char *__haystack, __const char *__needle)
     __THROW __attribute_pure__;


/* Divide S into tokens separated by characters in DELIM.  */
extern char *strtok (char *__restrict __s, __const char *__restrict __delim)
     __THROW;
__END_NAMESPACE_STD

/* Divide S into tokens separated by characters in DELIM.  Information
   passed between calls are stored in SAVE_PTR.  */
extern char *__strtok_r (char *__restrict __s,
			 __const char *__restrict __delim,
			 char **__restrict __save_ptr) __THROW;
#if defined __USE_POSIX || defined __USE_MISC
extern char *strtok_r (char *__restrict __s, __const char *__restrict __delim,
		       char **__restrict __save_ptr) __THROW;
#endif

#ifdef __USE_GNU
/* Similar to `strstr' but this function ignores the case of both strings.  */
extern char *strcasestr (__const char *__haystack, __const char *__needle)
     __THROW __attribute_pure__;
#endif

#ifdef __USE_GNU
/* Find the first occurrence of NEEDLE in HAYSTACK.
   NEEDLE is NEEDLELEN bytes long;
   HAYSTACK is HAYSTACKLEN bytes long.  */
extern void *memmem (__const void *__haystack, size_t __haystacklen,
		     __const void *__needle, size_t __needlelen)
     __THROW __attribute_pure__;

/* Copy N bytes of SRC to DEST, return pointer to bytes after the
   last written byte.  */
extern void *__mempcpy (void *__restrict __dest,
			__const void *__restrict __src, size_t __n) __THROW;
extern void *mempcpy (void *__restrict __dest,
		      __const void *__restrict __src, size_t __n) __THROW;
#endif


__BEGIN_NAMESPACE_STD
#ifndef __SYSC__
/* Return the length of S.  */
extern size_t strlen (__const char *__s) __THROW __attribute_pure__;
#endif
__END_NAMESPACE_STD


__BEGIN_NAMESPACE_STD
/* Return a string describing the meaning of the `errno' code in ERRNUM.  */
extern char *strerror (int __errnum) __THROW;
__END_NAMESPACE_STD
#if defined __USE_XOPEN2K || defined __USE_MISC
/* Reentrant version of `strerror'.  If a temporary buffer is required, at
   most BUFLEN bytes of BUF will be used.  */
extern char *strerror_r (int __errnum, char *__buf, size_t __buflen) __THROW;
#endif

/* We define this function always since `bzero' is sometimes needed when
   the namespace rules does not allow this.  */
extern void __bzero (void *__s, size_t __n) __THROW;

#ifdef __USE_BSD
/* Copy N bytes of SRC to DEST (like memmove, but args reversed).  */
extern void bcopy (__const void *__src, void *__dest, size_t __n) __THROW;

/* Set N bytes of S to 0.  */
extern void bzero (void *__s, size_t __n) __THROW;

/* Compare N bytes of S1 and S2 (same as memcmp).  */
extern int bcmp (__const void *__s1, __const void *__s2, size_t __n)
     __THROW __attribute_pure__;

/* Find the first occurrence of C in S (same as strchr).  */
extern char *index (__const char *__s, int __c) __THROW __attribute_pure__;

/* Find the last occurrence of C in S (same as strrchr).  */
extern char *rindex (__const char *__s, int __c) __THROW __attribute_pure__;

/* Return the position of the first bit set in I, or 0 if none are set.
   The least-significant bit is position 1, the most-significant 32.  */
extern int ffs (int __i) __THROW __attribute__ ((__const__));

/* The following two functions are non-standard but necessary for non-32 bit
   platforms.  */
# ifdef	__USE_GNU
extern int ffsl (long int __l) __THROW __attribute__ ((__const__));
#  ifdef __GNUC__
__extension__ extern int ffsll (long long int __ll)
     __THROW __attribute__ ((__const__));
#  endif
# endif

/* Compare S1 and S2, ignoring case.  */
extern int strcasecmp (__const char *__s1, __const char *__s2)
     __THROW __attribute_pure__;

/* Compare no more than N chars of S1 and S2, ignoring case.  */
extern int strncasecmp (__const char *__s1, __const char *__s2, size_t __n)
     __THROW __attribute_pure__;
#endif /* Use BSD.  */

#ifdef	__USE_GNU
/* Again versions of a few functions which use the given locale instead
   of the global one.  */
extern int strcasecmp_l (__const char *__s1, __const char *__s2,
			 __locale_t __loc) __THROW __attribute_pure__;

extern int strncasecmp_l (__const char *__s1, __const char *__s2,
			  size_t __n, __locale_t __loc)
     __THROW __attribute_pure__;
#endif

#ifdef	__USE_BSD
/* Return the next DELIM-delimited token from *STRINGP,
   terminating it with a '\0', and update *STRINGP to point past it.  */
extern char *strsep (char **__restrict __stringp,
		     __const char *__restrict __delim) __THROW;
#endif

#ifdef	__USE_GNU
/* Compare S1 and S2 as strings holding name & indices/version numbers.  */
extern int strverscmp (__const char *__s1, __const char *__s2)
     __THROW __attribute_pure__;

/* Return a string describing the meaning of the signal number in SIG.  */
extern char *strsignal (int __sig) __THROW;

/* Copy SRC to DEST, returning the address of the terminating '\0' in DEST.  */
extern char *__stpcpy (char *__restrict __dest, __const char *__restrict __src)
     __THROW;
extern char *stpcpy (char *__restrict __dest, __const char *__restrict __src)
     __THROW;

/* Copy no more than N characters of SRC to DEST, returning the address of
   the last character written into DEST.  */
extern char *__stpncpy (char *__restrict __dest,
			__const char *__restrict __src, size_t __n) __THROW;
extern char *stpncpy (char *__restrict __dest,
		      __const char *__restrict __src, size_t __n) __THROW;

/* Sautee STRING briskly.  */
extern char *strfry (char *__string) __THROW;

/* Frobnicate N bytes of S.  */
extern void *memfrob (void *__s, size_t __n) __THROW;

# ifndef basename
/* Return the file name within directory of FILENAME.  We don't
   declare the function if the `basename' macro is available (defined
   in <libgen.h>) which makes the XPG version of this function
   available.  */
extern char *basename (__const char *__filename) __THROW;
# endif
#endif

#ifdef __SYSC__
/* Compiler-defined builtin's */

#ifdef __cplusplus
extern "builtin" {
#else
#pragma linkage(__memcpy, builtin)
#pragma linkage(__memset, builtin)
#pragma linkage(__memcmp, builtin)
#pragma linkage(__memchr, builtin)
#pragma linkage(__strcpy, builtin)
#pragma linkage(__strlen, builtin)
#pragma linkage(__strcmp, builtin)
#pragma linkage(__strcat, builtin)
#pragma linkage(__strchr, builtin)
#pragma linkage(__strrchr, builtin)
#pragma linkage(__strncat, builtin)
#pragma linkage(__strncpy, builtin)
#pragma linkage(__strncmp, builtin)
#pragma linkage(__strpbrk, builtin)
#endif

void		 *__memcpy (void * __restrict, const void * __restrict, size_t);
void		 *__memset (void *, int, size_t);
int		  __memcmp (const void *, const void *, size_t);
void		 *__memchr (const void *, int, size_t);

char		 *__strcpy (char * __restrict, const char * __restrict);
size_t		  __strlen (const char *);
int		  __strcmp (const char *, const char *);
char		 *__strcat (char * __restrict, const char * __restrict);
char		 *__strchr (const char *, int);
char		 *__strrchr (const char *, int);

char		 *__strncat (char * __restrict, const char * __restrict, size_t);
int		  __strncmp (const char *, const char *, size_t);
char		 *__strncpy (char * __restrict, const char * __restrict, size_t);
char		 *__strpbrk (const char *, const char *);
#ifdef __cplusplus
}
#endif

/* Define the functions that have builtin equivalents, so the */
/*  builtins will be used when appropriate. */
#ifdef __cplusplus
  /* For C++ - define inline functions which invoke the appropriate */
  /*  compiler builtin functions */
inline void * memchr (const void *x, int y, size_t z)
       { return __memchr(x, y, z); }
inline void * memcpy (void *x, const void *y, size_t z)
       { return __memcpy(x, y, z); }
inline int    memcmp (const void *x, const void *y, size_t z)
       { return __memcmp(x, y, z); }
inline void * memset (void *x, int y, size_t z)
       { return __memset(x, y, z); }

inline size_t strlen (const char *x)
       { return __strlen(x);       }
inline char * strcat (char *x, const char *y)
       { return __strcat(x, y);    }
inline char * strcpy (char *x, const char *y)
       { return __strcpy(x, y);    }
inline int    strcmp (const char *x, const char *y)
       { return __strcmp(x, y);    }
inline char * strchr (const char *s, int c)
       { return __strchr(s, c);    }
inline char * strrchr(const char *s, int c)
       { return __strrchr(s, c);   }
inline char * strpbrk (const char *s, const char *charset)
       { return __strpbrk(s, charset); }

inline char * strncat (char *s, const char *x, size_t z)
       { return __strncat(s, x, z); }
inline int strncmp (const char *x, const char *y, size_t z)
       { return __strncmp(x, y, z); }
inline char * strncpy (char *s, const char *x, size_t z)
       { return __strncpy(s, x, z); }
#else


 /* For `C' - use #define macros to handle this */
#define memset(a,b,c) __memset(a,b,c)
#define memcpy(a,b,c) __memcpy(a,b,c)
#define memchr(a,b,c) __memchr(a,b,c)
#define memcmp(a,b,c) __memcmp(a,b,c)

#define strlen(a)     __strlen(a)
#define strcat(a,b)   __strcat(a,b)
#define strcpy(a,b)   __strcpy(a,b)
#define strcmp(a,b)   __strcmp(a,b)
#define strchr(a,b)   __strchr(a,b)
#define strrchr(a,b)  __strrchr(a,b)
#define strpbrk(a,b)  __strpbrk(a,b)

#define		 strncat(a,b,c) __strncat(a,b,c)
#define		 strncmp(a,b,c) __strncmp(a,b,c)
#define		 strncpy(a,b,c) __strncpy(a,b,c)
#endif

#endif /* __SYSC__ */


#if defined __GNUC__ && __GNUC__ >= 2
# if defined __OPTIMIZE__ && !defined __OPTIMIZE_SIZE__ \
     && !defined __NO_INLINE__ && !defined __cplusplus
/* When using GNU CC we provide some optimized versions of selected
   functions from this header.  There are two kinds of optimizations:

   - machine-dependent optimizations, most probably using inline
     assembler code; these might be quite expensive since the code
     size can increase significantly.
     These optimizations are not used unless the symbol
	__USE_STRING_INLINES
     is defined before including this header.

   - machine-independent optimizations which do not increase the
     code size significantly and which optimize mainly situations
     where one or more arguments are compile-time constants.
     These optimizations are used always when the compiler is
     taught to optimize.

   One can inhibit all optimizations by defining __NO_STRING_INLINES.  */

/* Get the machine-dependent optimizations (if any).  */
#  include <bits/string.h>

/* These are generic optimizations which do not add too much inline code.  */
#  include <bits/string2.h>
# endif
#endif

__END_DECLS

#endif /* string.h  */
